<?php

namespace Drupal\cache_debug;

use Drupal\cache_debug\CacheDebugLogger\CacheDebugLoggerInterface;
use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\DestructableInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Cache debug service.
 */
class CacheDebug implements CacheDebugInterface, DestructableInterface {

  /**
   * The module settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $settings;

  /**
   * The cache debug logger manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected PluginManagerInterface $cacheDebugLoggerManager;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * The cache debug logger instances.
   *
   * @var \Drupal\cache_debug\CacheDebugLogger\CacheDebugLoggerInterface[]|null[]
   */
  protected array $cacheDebugLoggers = [];

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Component\Plugin\PluginManagerInterface $cache_debug_logger_manager
   *   The cache debug logger manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(ConfigFactoryInterface $config_factory, PluginManagerInterface $cache_debug_logger_manager, RequestStack $request_stack) {
    $this->settings = $config_factory->get('cache_debug.settings');
    $this->cacheDebugLoggerManager = $cache_debug_logger_manager;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public function logResponseCacheTags(array $cache_tags, Request $request): void {
    $this->getCacheDebugLogger('response_logger')?->logCacheTags(
      CacheDebugLoggerInterface::TYPE_RESPONSE,
      $cache_tags,
      'Response cache tags for ' . $this->describeRequest($request)
    );
  }

  /**
   * {@inheritdoc}
   */
  public function logInvalidatedCacheTags(array $cache_tags): void {
    $description = NULL;
    $request = $this->requestStack->getMainRequest();

    if ($request !== NULL) {
      $description = 'Invalidated cache tags for ' . $this->describeRequest($request);
    }
    elseif (!empty($_SERVER['argv'])) {
      $argv = $_SERVER['argv'];
      $argv[0] = basename($argv[0]);
      $description = 'Invalidated cache tags on ' . implode(' ', $argv);
    }

    $this->getCacheDebugLogger('invalidated_logger')?->logCacheTags(
      CacheDebugLoggerInterface::TYPE_INVALIDATED,
      $cache_tags,
      $description
    );
  }

  /**
   * {@inheritdoc}
   */
  public function destruct() {
    foreach ($this->cacheDebugLoggers as $cache_debug_logger) {
      if ($cache_debug_logger instanceof DestructableInterface) {
        $cache_debug_logger->destruct();
      }
    }
  }

  /**
   * Generate a description for a request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return string
   *   The description.
   */
  protected function describeRequest(Request $request): string {
    return $request->getRealMethod() . ' ' . $request->getRequestUri();
  }

  /**
   * Get the response or invalidated cache debug logger.
   *
   * @param string $config_key
   *   The key of the config that holds the plugin ID.
   *
   * @return \Drupal\cache_debug\CacheDebugLogger\CacheDebugLoggerInterface|null
   *   The plugin instance or NULL if not available.
   */
  protected function getCacheDebugLogger(string $config_key): ?CacheDebugLoggerInterface {
    if (array_key_exists($config_key, $this->cacheDebugLoggers)) {
      return $this->cacheDebugLoggers[$config_key];
    }

    $this->cacheDebugLoggers[$config_key] = NULL;

    $plugin_id = $this->settings->get($config_key);

    if (empty($plugin_id)) {
      return NULL;
    }

    try {
      $instance = $this->cacheDebugLoggerManager->createInstance($plugin_id);
    }
    catch (\Exception $ex) {
      return NULL;
    }

    if (!$instance instanceof CacheDebugLoggerInterface || !$instance->isAvailable()) {
      return NULL;
    }

    $this->cacheDebugLoggers[$config_key] = $instance;

    return $instance;
  }

}
