<?php

namespace Drupal\cache_debug\CacheDebugLogger;

use Drupal\Core\DestructableInterface;

/**
 * Base class for cache debug loggers that group invalidated cache tags.
 *
 * This class groups invalidated cache tags by description to minimize
 * the number of log entries.
 */
abstract class GroupInvalidatedCacheDebugLogger extends CacheDebugLoggerBase implements DestructableInterface {

  /**
   * List of invalidated cache tags.
   *
   * @var array[]
   */
  protected array $invalidatedCacheTags = [];

  /**
   * {@inheritdoc}
   */
  public function logCacheTags(string $type, array $cache_tags, ?string $description = NULL): void {
    if ($type !== self::TYPE_INVALIDATED) {
      $this->writeCacheTagsLog($type, $cache_tags, $description);
      return;
    }

    // Group the invalidated cache tags by description.
    $description = $description ?? '';

    if (!empty($this->invalidatedCacheTags[$description])) {
      $this->invalidatedCacheTags[$description] = array_merge($this->invalidatedCacheTags[$description], $cache_tags);
    }
    else {
      $this->invalidatedCacheTags[$description] = $cache_tags;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function destruct() {
    // Write the invalidated cache tags on destruction.
    foreach ($this->invalidatedCacheTags as $description => $cache_tags) {
      if ($description === '') {
        $description = NULL;
      }

      $this->writeCacheTagsLog(self::TYPE_INVALIDATED, array_unique($cache_tags), $description);
    }
  }

  /**
   * Write a cache tags log.
   *
   * @param string $type
   *   The type of log.
   * @param string[] $cache_tags
   *   The cache tags.
   * @param string|null $description
   *   An optional description.
   */
  abstract protected function writeCacheTagsLog(string $type, array $cache_tags, string $description): void;

}
