<?php

namespace Drupal\cache_debug\EventSubscriber;

use Drupal\cache_debug\CacheDebugInterface;
use Drupal\Core\Cache\CacheableResponseInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Log the cache tags of a response.
 */
class LogCacheTagsSubscriber implements EventSubscriberInterface {

  /**
   * The cache debug service.
   *
   * @var \Drupal\cache_debug\CacheDebugInterface
   */
  protected CacheDebugInterface $cacheDebug;

  /**
   * Class constructor.
   *
   * @param \Drupal\cache_debug\CacheDebugInterface $cache_debug
   *   The cache debug service.
   */
  public function __construct(CacheDebugInterface $cache_debug) {
    $this->cacheDebug = $cache_debug;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      KernelEvents::RESPONSE => [
        'logResponseCacheTags',
      ],
    ];
  }

  /**
   * Log cache tags of the response.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The response event.
   */
  public function logResponseCacheTags(ResponseEvent $event): void {
    // Leave if not the main request.
    if (!$event->isMainRequest()) {
      return;
    }

    // Leave if the response ain't cacheable.
    $response = $event->getResponse();

    if (!$response instanceof CacheableResponseInterface) {
      return;
    }

    // Get the cache tags.
    $cache_tags = $response->getCacheableMetadata()->getCacheTags();

    if (empty($cache_tags)) {
      return;
    }

    // Log them.
    $this->cacheDebug->logResponseCacheTags($cache_tags, $event->getRequest());
  }

}
