<?php

namespace Drupal\cache_debug\Form;

use Drupal\cache_debug\CacheDebugLogger\CacheDebugLoggerInterface;
use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManager;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form to edit the settings.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The cache debug logger manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected PluginManagerInterface $cacheDebugLoggerManager;

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Config\TypedConfigManager $typed_config_manager
   *   The typed config manager.
   * @param \Drupal\Component\Plugin\PluginManagerInterface $cache_debug_logger_manager
   *   The cache debug logger manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, TypedConfigManager $typed_config_manager, PluginManagerInterface $cache_debug_logger_manager) {
    parent::__construct($config_factory, $typed_config_manager);

    $this->cacheDebugLoggerManager = $cache_debug_logger_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('plugin.manager.cache_debug_logger')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['cache_debug.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'cache_debug_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('cache_debug.settings');
    $options = $this->getLoggerOptions();

    $form['response_logger'] = [
      '#type' => 'radios',
      '#title' => $this->t('Response logger'),
      '#options' => $options,
      '#default_value' => $config->get('response_logger') ?? '_none',
      '#required' => TRUE,
    ];

    $form['invalidated_logger'] = [
      '#type' => 'radios',
      '#title' => $this->t('Invalidated logger'),
      '#options' => $options,
      '#default_value' => $config->get('invalidated_logger') ?? '_none',
      '#required' => TRUE,
    ];

    $form['log_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('File log path'),
      '#description' => $this->t('URI or path where file logs should be saved.'),
      '#default_value' => $config->get('log_path'),
      '#required' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $response_logger = $form_state->getValue('response_logger');
    $response_logger = $response_logger === '_none' ? NULL : $response_logger;

    $invalidated_logger = $form_state->getValue('invalidated_logger');
    $invalidated_logger = $invalidated_logger === '_none' ? NULL : $invalidated_logger;

    $this->config('cache_debug.settings')
      ->set('response_logger', $response_logger)
      ->set('invalidated_logger', $invalidated_logger)
      ->set('log_path', rtrim($form_state->getValue('log_path'), '/'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Get the logger options.
   *
   * @return string[]
   *   The logger options.
   */
  protected function getLoggerOptions(): array {
    $options = ['_none' => $this->t('None')];

    foreach ($this->cacheDebugLoggerManager->getDefinitions() as $plugin_id => $definition) {
      try {
        $instance = $this->cacheDebugLoggerManager->createInstance($plugin_id);
      }
      catch (\Exception $ex) {
        continue;
      }

      if ($instance instanceof CacheDebugLoggerInterface && $instance->isAvailable()) {
        $options[$plugin_id] = (string) $definition['label'];
      }
    }

    return $options;
  }

}
