<?php

namespace Drupal\cache_debug\Plugin\CacheDebugLogger;

use Drupal\cache_debug\CacheDebugLogger\CacheDebugLoggerBase;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\purge\CacheTagMinificatorInterface;
use Sentry\State\Scope;
use Symfony\Component\DependencyInjection\ContainerInterface;
use function Sentry\configureScope;

/**
 * Provides the Sentry.io cache debug logger plugin.
 *
 * @CacheDebugLogger(
 *   id = "sentry",
 *   label = @Translation("Sentry.io"),
 * )
 */
class Sentry extends CacheDebugLoggerBase implements ContainerFactoryPluginInterface {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The cache tag minificator.
   *
   * @var \Drupal\purge\CacheTagMinificatorInterface
   */
  protected CacheTagMinificatorInterface $cacheTagMinificator;

  /**
   * Class constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\purge\CacheTagMinificatorInterface $cache_tag_minificator
   *   The cache tag minificator.
   */
  public function __construct(array $configuration, string $plugin_id, array $plugin_definition, ModuleHandlerInterface $module_handler, ?CacheTagMinificatorInterface $cache_tag_minificator = NULL) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->moduleHandler = $module_handler;

    if ($cache_tag_minificator !== NULL) {
      $this->cacheTagMinificator = $cache_tag_minificator;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('module_handler'),
      $container->get('purge.cache_tag_minificator', ContainerInterface::NULL_ON_INVALID_REFERENCE)
    );
  }

  /**
   * {@inheritdoc}
   */
  public function isAvailable(): bool {
    return $this->moduleHandler->moduleExists('raven');
  }

  /**
   * {@inheritdoc}
   */
  public function logCacheTags(string $type, array $cache_tags, ?string $description = NULL): void {
    configureScope(function (Scope $scope) use ($type, $cache_tags): void {
      foreach ($cache_tags as $cache_tag) {
        $scope->setTag($this->getTagKey($type, $cache_tag), $cache_tag);
      }
    });
  }

  /**
   * Get the key for a cache tag.
   *
   * @param string $type
   *   The type of log.
   * @param string $cache_tag
   *   The cache tag.
   *
   * @return string
   *   The tag key.
   */
  protected function getTagKey(string $type, string $cache_tag): string {
    $prefix = $type[0] . 'ct.';

    if (isset($this->cacheTagMinificator)) {
      return $prefix . str_replace('.', '_', $this->cacheTagMinificator->minifyCacheTag($cache_tag));
    }

    return $prefix . substr(md5($cache_tag), 0, 4);
  }

}
